// ignore_for_file: non_constant_identifier_names, camel_case_extensions, empty_catches

import 'dart:async';

// import 'package:galaxeus_lib/galaxeus_lib.dart';
import 'package:telegram_client/telegram_client/scheme/json_dart.dart'
    as scheme;
import 'package:telegram_client/telegram_client/telegram_client.dart';
import 'package:telegram_client/util/util.dart';

/// method MessageData
extension MessageDataDataOn on TelegramClient {
  Future<Map> message_Message({
    required Map message,
    required TelegramClientData telegramClientData,
    bool is_skip_reply_message = false,
  }) async {
    Map message_thread_json = {};
    if (message["message_thread_id"] is int &&
        message["message_thread_id"] > 0) {
      message_thread_json["message_thread_id"] = message["message_thread_id"];
    }
    Map message_sender_chat_json = {};

    if (message["sender_id"] is Map) {
      if (message["sender_id"]["@type"] == "messageSenderUser") {
        if (message["sender_id"]["user_id"] is int) {
          message_sender_chat_json["id"] = message["sender_id"]["user_id"];
          var res = await request(
            parameters: {
              "@type": "getUser",
              "user_id": message_sender_chat_json["id"],
            },
            telegramClientData: telegramClientData,
          );

          message_sender_chat_json.addAll(res.rawData);
        }
      }
      if (message["sender_id"]["@type"] == "messageSenderChat") {
        if (message["sender_id"]["chat_id"] is int) {
          message_sender_chat_json["id"] = message["sender_id"]["chat_id"];
          var res = await request(
            parameters: {
              "@type": "getChat",
              "chat_id": message_sender_chat_json["id"],
            },
            telegramClientData: telegramClientData,
          );

          message_sender_chat_json.addAll(res.rawData);
        }
      }
    }
    Map message_chat_json = {};

    if (message["chat_id"] is int) {
      message_chat_json["id"] = message["chat_id"];
      var res = await request(
        parameters: {
          "@type": "getChat",
          "chat_id": message_chat_json["id"],
        },
        telegramClientData: telegramClientData,
      );
      message_chat_json.addAll(res.rawData);
    }
    if (telegramClientData.is_bot == false) {
      message_sender_chat_json
          .removeWhere((key, value) => key == "phone_number");
      message_chat_json.removeWhere((key, value) => key == "phone_number");
    }
    Map msg = {
      "@type": "message",
      "id": message["id"],
      "message_id": TgUtils().messageTdlibToApi(message["id"]),
      ...message_thread_json,
      "from": message_sender_chat_json,
      "chat": message_chat_json,
      "date": message["date"],
      "is_outgoing": (message["is_outgoing"] == true),
      "content_type": "",
    };

    if (message["is_channel_post"] == true) {
      msg["chat"]["type"] = "channel";
    }

    if (message["reply_to"] is Map) {
      if (is_skip_reply_message == false) {
        if (message["reply_to"]["@type"] == "messageReplyToMessage") {
          int message_reply_to_chat_id = message["reply_to"]["chat_id"];
          int message_reply_to_message_id = message["reply_to"]["message_id"];
          if (message_reply_to_message_id > 0) {
            scheme.JsonScheme message_reply_to = await invoke(
              parameters: {
                "@type": "getRepliedMessage",
                "chat_id": message_reply_to_chat_id,
                "message_id": message["id"],
              },
              telegramClientData: telegramClientData,
            );

            msg["reply_to_message"] = await message_Message(
              message: message_reply_to.rawData,
              telegramClientData: telegramClientData,
              is_skip_reply_message: true,
            );
          }
        }
      }
    }

    if (message["edit_date"] is int && message["edit_date"] > 0) {
      msg["edit_date"] = message["edit_date"];
    }

    if (message["forward_info"] is Map) {
      Map forwardInfo = message["forward_info"];
      if (forwardInfo["date"] is int) {
        msg["forward_date"] = forwardInfo["date"];
      }
    }

    if (message["author_signature"] is String &&
        (message["author_signature"] as String).isNotEmpty) {
      msg["author_signature"] = message["author_signature"];
    }
    if (message["content"] is Map) {
      // ignore: unused_local_variable
      List text_entities = [];
      Map message_content = message["content"];
      if (message_content["@type"] == "messageText") {
        msg["content_type"] = "text";
        if (message_content["text"] is Map) {
          msg["text"] = message_content["text"]["text"];
          if (message["content"]["text"]["entities"] is List) {
            text_entities = message["content"]["text"]["entities"];
          }
        }
      }

      if (message_content["@type"] == "messageAnimation") {
        msg["content_type"] = "animation";

        /// http://0.0.0.0:8080/classtd_1_1td__api_1_1animation.html
        Map animation = {};
        if (message["animation"] is Map) {
          if (message["animation"]["animation"] is Map) {
            Map message_animation = message["animation"]["animation"];
            animation["id"] = message_animation["id"];
            if (message_animation["local"] is Map) {
              animation["file_path"] = message_animation["local"]["path"];
              animation["is_downloading_active"] =
                  message_animation["local"]["is_downloading_active"];
              animation["is_downloading_completed"] =
                  message_animation["local"]["is_downloading_completed"];
            }
            if (message_animation["remote"] is Map) {
              animation["file_id"] = message_animation["remote"]["id"];
              animation["file_unique_id"] =
                  message_animation["remote"]["unique_id"];
              animation["file_size"] = message_animation["remote"]["size"];
            }
          }
          animation["width"] = message["animation"]["width"];
          animation["height"] = message["animation"]["height"];
          animation["duration"] = message["animation"]["duration"];
          // Map msg_content_thumbnail = {};
          animation["file_name"] = message["animation"]["file_name"];
          animation["mime_type"] = message["animation"]["mime_type"];
        }
        msg["animation"] = animation;
      }

      if (message_content["@type"] == "messagePhoto") {
        // http://0.0.0.0:8080/classtd_1_1td__api_1_1animation.html
        List photos = [];
        if (message_content["photo"] is Map) {
          List photo_sizes = message_content["photo"]["sizes"];

          for (var i = 0; i < photo_sizes.length; i++) {
            dynamic photo_data = photo_sizes[i];
            if (photo_data is Map) {
              if (photo_data["photo"] is Map) {
                Map jsonDataPhoto = {};

                Map message_photo = photo_data["photo"];
                jsonDataPhoto["id"] = message_photo["id"];
                if (message_photo["local"] is Map) {
                  jsonDataPhoto["file_path"] = message_photo["local"]["path"];
                  jsonDataPhoto["is_downloading_active"] =
                      message_photo["local"]["is_downloading_active"];
                  jsonDataPhoto["is_downloading_completed"] =
                      message_photo["local"]["is_downloading_completed"];
                }
                if (message_photo["remote"] is Map) {
                  jsonDataPhoto["file_id"] = message_photo["remote"]["id"];
                  jsonDataPhoto["file_unique_id"] =
                      message_photo["remote"]["unique_id"];
                  jsonDataPhoto["file_size"] = message_photo["remote"]["size"];
                }
                photos.add(jsonDataPhoto);
              }
            }
          }
        }
        msg["photo"] = photos;
      }

      if (message_content["caption"] is Map) {
        if (message_content["caption"]["text"] is String &&
            (message_content["caption"]["text"] as String).isNotEmpty) {
          msg["caption"] = message_content["caption"]["text"];
        }
      }

      if (message["content"]["@type"] == "messageVideo") {
        msg["content_type"] = "video";
        if (message["content"]["video"] is Map) {
          if (message["content"]["video"]["@type"] == "video") {
            var jsonVideo = {};
            var contentVideo = message["content"]["video"];
            jsonVideo["duration"] = contentVideo["duration"];
            jsonVideo["height"] = contentVideo["height"];
            jsonVideo["file_name"] = contentVideo["file_name"];
            jsonVideo["mime_type"] = contentVideo["mime_type"];
            try {
              if (message["content"]["video"]["thumbnail"] != null &&
                  message["content"]["video"]["thumbnail"]["@type"]
                          .toString()
                          .toLowerCase() ==
                      "thumbnail") {
                var contentThumb = contentVideo["thumbnail"];
                var jsonThumb = {};
                jsonVideo["thumb"] = jsonThumb;
                jsonThumb["file_id"] = contentThumb["file"]["remote"]["id"];
                jsonThumb["file_unique_id"] =
                    contentThumb["file"]["remote"]["unique_id"];
                jsonThumb["file_size"] = contentThumb["file"]["size"];
                jsonThumb["width"] = contentThumb["width"];
                jsonThumb["height"] = contentThumb["height"];
              }
            } catch (e) {}
            jsonVideo["file_id"] = contentVideo["video"]["remote"]["id"];
            jsonVideo["file_size"] = contentVideo["video"]["size"];
            msg["video"] = jsonVideo;
          }
        }
      }

      // http://localhost:8080/classtd_1_1td__api_1_1audio.html
      if (message["content"]["@type"] == "messageAudio") {
        var contentType = "audio";
        msg["content_type"] = "audio";
        if (message["content"]["audio"] is Map) {
          if (message["content"]["audio"]["@type"] == "audio") {
            var jsonContent = {};
            var contentUpdate = message["content"][contentType];

            jsonContent["id"] = contentUpdate[contentType]["id"];
            jsonContent["file_path"] =
                contentUpdate[contentType]["local"]["path"];
            jsonContent["is_downloading_active"] =
                contentUpdate[contentType]["local"]["is_downloading_active"];
            jsonContent["is_downloading_completed"] =
                contentUpdate[contentType]["local"]["is_downloading_completed"];

            jsonContent["file_id"] = contentUpdate[contentType]["remote"]["id"];
            jsonContent["file_unique_id"] =
                contentUpdate[contentType]["remote"]["unique_id"];
            jsonContent["duration"] = contentUpdate["duration"];
            jsonContent["performer"] = contentUpdate["performer"];
            jsonContent["file_name"] = contentUpdate["file_name"];
            jsonContent["title"] = contentUpdate["title"];
            jsonContent["mime_type"] = contentUpdate["mime_type"];
            jsonContent["file_size"] = contentUpdate[contentType]["size"];
            msg[contentType] = jsonContent;
          }
        }
      }
      // http://localhost:8080/classtd_1_1td__api_1_1file.html
      // https://core.telegram.org/bots/api#contact
      if (message["content"]["@type"] == "messageContact") {
        var contentType = "contact";
        msg["content_type"] = contentType;
        if (message["content"][contentType] is Map) {
          if (message["content"][contentType]["@type"] == contentType) {
            var jsonContent = {};
            var contentUpdate = message["content"][contentType];
            jsonContent["phone_number"] = contentUpdate["phone_number"];
            jsonContent["first_name"] = contentUpdate["first_name"];
            jsonContent["last_name"] = contentUpdate["last_name"];
            jsonContent["user_id"] = contentUpdate["user_id"];
            jsonContent["vcard"] = contentUpdate["vcard"];
            msg[contentType] = jsonContent;
          }
        }
      }

      if (message["content"]["@type"] == "messagePoll") {
        var contentType = "poll";
        msg["content_type"] = contentType;
        if (message["content"][contentType] is Map) {
          if (message["content"][contentType]["@type"] == contentType) {
            var jsonContent = {};
            var contentUpdate = message["content"][contentType];
            jsonContent["id"] = contentUpdate["id"];
            jsonContent["question"] = contentUpdate["question"];
            jsonContent["options"] = contentUpdate["options"];
            jsonContent["total_voter_count"] =
                contentUpdate["total_voter_count"];
            jsonContent["recent_voter_user_ids"] =
                contentUpdate["recent_voter_user_ids"];
            jsonContent["is_anonymous"] = contentUpdate["is_anonymous"];
            jsonContent["type"] = contentUpdate["type"];
            jsonContent["open_period"] = contentUpdate["open_period"];
            jsonContent["close_date"] = contentUpdate["close_date"];
            jsonContent["is_closed"] = contentUpdate["is_closed"];
            msg[contentType] = jsonContent;
          }
        }
      }

      if (message["content"]["@type"] == "messageDocument") {
        var contentType = "document";
        msg["content_type"] = contentType;
        if (message["content"][contentType] is Map) {
          if (message["content"][contentType]["@type"] == contentType) {
            var jsonContent = {};
            var contentUpdate = message["content"][contentType];
            jsonContent["file_name"] = contentUpdate["file_name"];
            jsonContent["mime_type"] = contentUpdate["mime_type"];

            jsonContent["file_id"] = contentUpdate[contentType]["remote"]["id"];
            jsonContent["unique_id"] =
                contentUpdate[contentType]["remote"]["unique_id"];
            jsonContent["file_size"] = contentUpdate[contentType]["size"];
            msg[contentType] = jsonContent;
          }
        }
      }

      if (message["content"]["@type"] == "messageSticker") {
        var contentType = "sticker";
        msg["content_type"] = contentType;
        if (message["content"][contentType] is Map) {
          if (message["content"][contentType]["@type"] == contentType) {
            var jsonContent = {};
            var contentUpdate = message["content"][contentType];
            jsonContent["set_id"] = contentUpdate["set_id"];

            try {
              var res = await invoke(
                parameters: {
                  "@type": "getStickerSet",
                  "set_id": int.tryParse(contentUpdate["set_id"]) ??
                      contentUpdate["set_id"],
                },
                telegramClientData: telegramClientData,
              );

              jsonContent["set_name"] = res["name"];
            } catch (e) {
              // print(e);
            }
            jsonContent["width"] = contentUpdate["width"];
            jsonContent["height"] = contentUpdate["height"];
            jsonContent["emoji"] = contentUpdate["emoji"];
            jsonContent["is_animated"] = contentUpdate["is_animated"];
            jsonContent["is_mask"] = contentUpdate["is_mask"];

            try {
              if (message["content"][contentType]["thumbnail"] != null &&
                  message["content"][contentType]["thumbnail"]["@type"]
                          .toString()
                          .toLowerCase() ==
                      "thumbnail") {
                var contentThumb = contentUpdate["thumbnail"];
                var jsonThumb = {};
                jsonThumb["file_id"] = contentThumb["file"]["remote"]["id"];
                jsonThumb["file_unique_id"] =
                    contentThumb["file"]["remote"]["unique_id"];
                jsonThumb["file_size"] = contentThumb["file"]["size"];
                jsonThumb["width"] = contentThumb["width"];
                jsonThumb["height"] = contentThumb["height"];
                jsonContent["thumb"] = jsonThumb;
              }
            } catch (e) {}

            jsonContent["file_id"] = contentUpdate[contentType]["remote"]["id"];
            jsonContent["file_unique_id"] =
                contentUpdate[contentType]["remote"]["unique_id"];
            jsonContent["file_size"] = contentUpdate[contentType]["size"];
            msg[contentType] = jsonContent;
          }
        }
      }

      if (message["content"]["@type"] == "messageVoiceNote") {
        var contentType = "voice_note";
        msg["content_type"] = contentType;
        if (message["content"][contentType] is Map) {
          if (message["content"][contentType]["@type"] == "voiceNote") {
            var jsonContent = {};
            var contentUpdate = message["content"][contentType];

            jsonContent["duration"] = contentUpdate["duration"];
            jsonContent["waveform"] = contentUpdate["waveform"];
            jsonContent["mime_type"] = contentUpdate["mime_type"];

            jsonContent["file_id"] = contentUpdate["voice"]["remote"]["id"];
            jsonContent["unique_id"] =
                contentUpdate["voice"]["remote"]["unique_id"];
            jsonContent["file_size"] = contentUpdate["voice"]["size"];
            msg["voice"] = jsonContent;
          }
        }
      }
    }

    if (message["reply_markup"] is Map) {
      Map? reply_markup = TgUtils.replyMarkupTdlibToTgApi(
        replyMarkup: message["reply_markup"],
      );
      if (reply_markup != null) {
        msg["reply_markup"] = reply_markup;
      }
    }
    message.forEach((key, value) {
      if (value is bool) {
        msg[key] = value;
      }
    });

    return msg;
  }

  /// method MessageData
  FutureOr<Map?> message_toJson({
    required Map update,
    required TelegramClientData telegramClientData,
  }) async {
    // http://0.0.0.0:8704/classtd_1_1td__api_1_1message.html
    if (update["@type"] == "updateNewMessage") {
      Map message = update["message"];
      if ((message["is_outgoing"] == true)) {
        if (telegramClientData.is_bot) {
          return null;
        }
      }
      // if (telegramClientData.is_bot == false) {
      //   DateTime dateTime = DateTime.now().copyWith(microsecond: 0, millisecond: 0);
      //   DateTime dateTimeMessage = DateTime.fromMillisecondsSinceEpoch((message["date"] * 1000));
      //   Duration duration = dateTime.difference(dateTimeMessage);

      //   if (duration > Duration(seconds: 10)) {
      //     return null;
      //   }
      // }

      Map msg = await message_Message(
        message: message,
        telegramClientData: telegramClientData,
      );

      if (msg["chat"]["type"] == "channel") {
        return {
          "@type": "updateChannelPost",
          "channel_post": msg,
        };
      } else {
        return {
          "@type": "updateMessage",
          "update_message": msg,
        };
      }
    }

    return null;
  }
}
